package grm;

import java.util.HashMap;
import java.lang.Double;

/* This enum is used to represent the resources monitored by LUPA (CPU and RAM) */
enum Resource { 
   CPU_USAGE("cpuUsage", 1.0),
   FREE_RAM("freeRam", 0.0);

   private String resourceName;
   // The don't care value means that the resource will not influence a LUPA
   // request. It's 1.0 for cpuUsage because we ask for hosts less usage than
   // this value, so less than 100%. It's 0.0 for freeRam because we ask for
   // hosts with at least 0.0 of RAM available
   private double dontCareValue;

   Resource(String name, double dontCare) {
      resourceName = name;
      dontCareValue = dontCare;
   }

   public String getName() {
      return resourceName;
   }

   public double getDontCareValue() {
      return dontCareValue;
   }
};

/* This class just encapsulates the requirements of an application, what means how much of CPU and
 * RAM the application needs, and for how long */

public class ApplicationRequirements {

   private HashMap<Resource, Double> resourceValues;

   private int hours;
   
   private static final double INVALID_VALUE = -1.0;

   public ApplicationRequirements() {
      hours = 0;
      resourceValues = new HashMap<Resource, Double>(Resource.values().length);
   }

   public Resource[] getAvailableResources() {
      return Resource.values();
   }

   public void setResourceValue(Resource r, double value) {
      resourceValues.put(r, Double.valueOf(value));
   }

   public void setHoursValue(int hours) {
      this.hours = hours;
   }

   public double getResourceValue(Resource r) {
      return (resourceValues.get(r)).doubleValue();
   }

   public int getHoursValue() {
      return hours;
   }

}
