package grm;

import java.util.ArrayList;
import java.util.Iterator;

import org.omg.CORBA.ORB;
import org.omg.CosTrading.Property;

import resourceProviders.Lrm;
import resourceProviders.LrmHelper;

/* This algorithm comunicates with each LRM, asking for the response to the canRunGridApplication consult. If
 * the answer is FALSE, then it removes this node from the selectedLrms array */

class CanRunGridApplicationSchedulingAlgorithm implements SchedulingAlgorithm {
   
   private ORB orb;

   public CanRunGridApplicationSchedulingAlgorithm(ORB orb) {
      this.orb = orb;
   }

   public void scheduleApplication(ApplicationRequirements applicationRequirements, ArrayList<Property[]> selectedLrms) {
      
      int hoursOfPrediction = applicationRequirements.getHoursValue();
      double freeCpuNeeded = applicationRequirements.getResourceValue(Resource.CPU_USAGE);
      /* Needed because we want the value for FREE cpu, not USED cpu */
      freeCpuNeeded = 1.0f - freeCpuNeeded;
      double freeRamNeeded = applicationRequirements.getResourceValue(Resource.FREE_RAM);

      for (Iterator it = selectedLrms.iterator(); it.hasNext(); ) {
         Property[] lrmProperties = (Property[]) it.next();
         String lrmIor = lrmProperties[0].value.extract_string();

         try {
            Lrm lrm = LrmHelper.narrow(orb.string_to_object(lrmIor));
            
            boolean canRunGridApplication = lrm.canRunGridApplication(freeCpuNeeded, freeRamNeeded, hoursOfPrediction);

            if (!canRunGridApplication) {
               it.remove();
            } 
         } catch (org.omg.CORBA.TRANSIENT transientException) {
				it.remove();
			}
      }
   }

   public String toString() {
      return "CAN RUN";
   }
}
