package grm;

import java.util.ArrayList;
import java.util.Iterator;

import java.lang.Double;

import org.omg.CORBA.ORB;
import org.omg.CosTrading.Property;

import resourceProviders.Lrm;
import resourceProviders.LrmHelper;

/* This algorithm consults the howLongCanRunGridApplication with each LRM, and then sorts the selectedLrms
 * array in a way that the firsts are those which the 'howLong' value return was the most */

class HowLongCanRunGridApplicationSchedulingAlgorithm implements SchedulingAlgorithm {
   
   private ORB orb;

   public HowLongCanRunGridApplicationSchedulingAlgorithm(ORB orb) {
      this.orb = orb;
   }

   public void scheduleApplication(ApplicationRequirements applicationRequirements, ArrayList<Property[]> selectedLrms) {
      
      int index = 0;

      int size =  selectedLrms.size();

      int hoursNeeded = applicationRequirements.getHoursValue();
      
      double freeCpuNeeded = applicationRequirements.getResourceValue(Resource.CPU_USAGE);
      /* Needed because we want the value for FREE cpu, not USED cpu */
      freeCpuNeeded = 1.0d - freeCpuNeeded;
      double freeRamNeeded = applicationRequirements.getResourceValue(Resource.FREE_RAM);

      ArrayList<Integer> howLong = new ArrayList<Integer>(size);

      for (Iterator it = selectedLrms.iterator(); it.hasNext(); ) {
         Property[] lrmProperties = (Property[]) it.next(); 
         String lrmIor = lrmProperties[0].value.extract_string();

         try {
            Lrm lrm = LrmHelper.narrow(orb.string_to_object(lrmIor));
            
            int howLongValue = lrm.howLongCanRunGridApplication(freeCpuNeeded, freeRamNeeded);
            
            // Checks, first, if the value return satisfies the application requirements
            if (howLongValue == 0 || howLongValue < hoursNeeded) {
               it.remove();
            } else {
               howLong.add(Integer.valueOf(howLongValue));
            }
         } catch (org.omg.CORBA.TRANSIENT transientException) {
				it.remove();
			}
      }

      size = selectedLrms.size();
      
      /* Sorts the selectedLrms array, with the informations in the
       * howLong array. A simple Selection Sort is used */
      Integer maxHours;
      int maxIndex;
      for (index = 0; index < size; index++) {
         maxHours = 0;
         maxIndex = -1;

         for (int j = index; j < size; j++) {
            Integer current = howLong.get(j);
            if (current != null && (current.compareTo(maxHours) > 0)) {
               maxIndex = j;
               maxHours = current;
            }
         }
         // Shifts the maximum hours to the correct position
         if (maxIndex != -1) {
            Property[] temp = selectedLrms.get(index);
            selectedLrms.set(index, selectedLrms.get(maxIndex));
            selectedLrms.set(maxIndex, temp);
            howLong.set(maxIndex, howLong.get(index));
         }
      }
   }

   public String toString() {
      return "HOW LONG";
   }
}
