package grm;

import dataTypes.ApplicationExecutionInformation;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/* This class does the parse of the 'constraints' string, received from the Asct. It will identify
 * the options 'cpuUsage', 'freeRam' and 'hours', and instantiate an object of the class ApplicationRequirements
 * with this options correctly setted.
 *
 * */

public class RequirementsAnalyser {

   private static double RESOURCE_NOT_FOUND = -1.0;
   
   ApplicationExecutionInformation applicationExecutionInformation;

   public RequirementsAnalyser(ApplicationExecutionInformation applicationExecutionInformation) {
      this.applicationExecutionInformation = applicationExecutionInformation;
   }

   public ApplicationRequirements getApplicationRequirements() {
      ApplicationRequirements applicationRequirements = new ApplicationRequirements();
      double resourceValue;
      int hours;

      for (Resource r : Resource.values()) {
         resourceValue = findResourceValueOnConstraints(r.getName());
         if (resourceValue == RESOURCE_NOT_FOUND) {
            resourceValue = r.getDontCareValue();
         }
         applicationRequirements.setResourceValue(r, resourceValue);
      }
      // Find the 'hours' field in the constraints
      resourceValue = findResourceValueOnConstraints("hours");
      if (resourceValue == RESOURCE_NOT_FOUND) {
         resourceValue = 0.0;
      }
      hours = (int)resourceValue;
      applicationRequirements.setHoursValue(hours);

      return applicationRequirements;
   }

   /* Tries to find, in the constraints String, the substring 'resourceName'. If it finds it,
    * it returns the value associate with this option */

   private double findResourceValueOnConstraints(String resourceName) {
      String constraints = applicationExecutionInformation.applicationConstraints;    
      double resourceValue;      
      // Get the position of the current resource name in the constraints String
      int position = constraints.indexOf(resourceName);
      
      resourceValue = RESOURCE_NOT_FOUND;
      if (position != -1) {
         // Searching for the first number after the resource name
         Pattern p = Pattern.compile(".+?(\\d+(\\.\\d+)?)");
         Matcher m = p.matcher(constraints.substring(position));
         if (m.find()) {
            try {
               resourceValue = Double.parseDouble(m.group(1).trim());
               if (resourceName.compareTo("cpuUsage") == 0) {
                  // This division has to be made because, in the Asct, the user is instructed to
                  // refer to the cpu usage in the range of 0-100%, but LUPA works with the range
                  // 0.0-1.0%
                  resourceValue = resourceValue / 100;
               }
            } catch (NumberFormatException nfe) {
               // nothing for now
            }
         }
      }
      return resourceValue;
   }
}
