package grm;

import java.util.ArrayList;
import java.util.Date;

import java.io.FileWriter;
import java.io.IOException;

import org.omg.CosTrading.Property;

/* This class just writes a log of every scheduling made. The log contains informations like the
 * time of the scheduling, the algorithm used, the hosts selected and if the scheduling was
 * successful or not, among other.
 *
 * */

public class SchedulingLogger {

   public static final int SUCCESSFUL_SCHEDULING = 0;

   public static final int UNSUCCESSFUL_SCHEDULING = 1;

   private static final String logFilename = "../scheduler.log";

   SchedulingAlgorithm schedulingAlgorithm;

   ApplicationRequirements applicationRequirements;

   ArrayList<Property[]> selectedLrms;

   public SchedulingLogger(SchedulingAlgorithm schedulingAlgorithm, ApplicationRequirements applicationRequirements, ArrayList<Property[]> selectedLrms) {
      this.schedulingAlgorithm = schedulingAlgorithm;
      this.applicationRequirements = applicationRequirements;
      this.selectedLrms = selectedLrms;
   }

   public void registerLog(int schedulingStatus) {
      FileWriter fileWriter;
      Date now = new Date();

      try {
         fileWriter = new FileWriter(logFilename, true);

         fileWriter.append("---------------------------------------------------------------------\n");
         fileWriter.append("[" + now.getTime() / 1000 + "] " + now.toString() + "\n");
         fileWriter.append("Algorithm: " + schedulingAlgorithm + "\n");
         fileWriter.append("Free CPU Needed: " + (1.0d - applicationRequirements.getResourceValue(Resource.CPU_USAGE)) + "\n");
         fileWriter.append("Free RAM Needed: " + applicationRequirements.getResourceValue(Resource.FREE_RAM) + "\n");
         fileWriter.append("Hours requested: " + applicationRequirements.getHoursValue() + "\n");
         fileWriter.append("LRMs selected: ");

         for (int i = 0; i < selectedLrms.size(); i++) { 
            Property[] lrmProperties = selectedLrms.get(i);
            String hostName = lrmProperties[1].value.extract_string();
            fileWriter.append(hostName + " ");
         }

         fileWriter.append("\nStatus: " + ((schedulingStatus == SUCCESSFUL_SCHEDULING) ? "Successful\n" : "Unsuccessful\n"));
         fileWriter.append("---------------------------------------------------------------------\n\n");

         fileWriter.close();

      } catch (IOException ioe) {
         System.err.println("Some problem occurred with the scheduler log file, named " + logFilename + ".");
         return;
      }
   }
}

